package go_swagger

import (
	"github.com/juju/errors"

	"git.home.m-and-m.ovh/mderasse/gouick/helpers"
	"git.home.m-and-m.ovh/mderasse/gouick/models"

	log "github.com/sirupsen/logrus"
)

// CreateProjectSkeleton will generate all the files needed to start a new project.
func (a APIType) CreateProjectSkeleton(path string, config *models.Config) error {
	log.Debugf("Starting %s  - %s", a.GetName(), helpers.GetCurrentFuncName())

	err := a.createDirectories(path)
	if err != nil {
		log.Error("Fail to create project directories")
		return errors.Trace(err)
	}

	// Generate Makefile
	err = a.generateMakefile(path, config)
	if err != nil {
		log.Error("Fail to generate Makefile")
		return errors.Trace(err)
	}
	// Generate Readme
	err = a.generateReadme(path, config)
	if err != nil {
		log.Error("Fail to generate Readme.md")
		return errors.Trace(err)
	}

	// Generate api yaml files
	err = a.generateAPIYamls(path, config)
	if err != nil {
		log.Error("Fail to generate API Yaml files")
		return errors.Trace(err)
	}

	// init Go Module
	err = a.initializeGoModule(path, config)
	if err != nil {
		log.Error("Fail to initialize Go Module")
		return errors.Trace(err)
	}

	// Launch Go Swagger
	err = a.executeGoSwagger(path, config)
	if err != nil {
		log.Error("Fail to Execute Go Swagger")
		return errors.Trace(err)
	}

	return nil
}
