package go_swagger

import (
	"os"
	"os/exec"
	"path/filepath"

	"github.com/juju/errors"

	"git.home.m-and-m.ovh/mderasse/gouick/helpers"
	"git.home.m-and-m.ovh/mderasse/gouick/models"

	log "github.com/sirupsen/logrus"
)

// executeGoSwagger will merge api yaml files and launch go-swagger generator.
func (a APIType) executeGoSwagger(path string, config *models.Config) error {
	log.Debugf("Starting %s - %s", a.GetName(), helpers.GetCurrentFuncName())

	// making sure that we are in the project directory
	if err := os.Chdir(path); err != nil {
		log.Error("Fail to move to the path")
		return errors.Trace(err)
	}

	// merge api files.
	// TODO: use filepath.Glob and avoid an ugly bin sh.
	cmd := exec.Command("/bin/sh", "-c", "swagger mixin --format=yaml --output=api-temp.yaml api.yaml api/*")
	output, err := cmd.CombinedOutput()
	if err != nil {
		log.Error("fail to execute 'swagger mixin --format=yaml --output=api-temp.yaml api.yaml api/*'")
		return errors.Trace(err)
	}

	log.Debugf("'swagger mixin --format=yaml --output=api-temp.yaml api.yaml api/*' returned the following output: %s", string(output))

	// cleanup some directory to avoid issues.
	for _, directory := range []string{"models", "restapi/operations"} {
		err := helpers.RemoveDirectoryContent(filepath.Join(path, directory))
		if err != nil {
			log.Errorf("fail to delete content of directory '%s'", directory)
			return errors.Trace(err)
		}
	}

	// generate server files
	cmd = exec.Command("swagger", "generate", "server", "--spec=api-temp.yaml")
	output, err = cmd.CombinedOutput()
	if err != nil {
		log.Error("fail to execute 'swagger generate server --spec=api-temp.yaml'")
		return errors.Trace(err)
	}

	log.Debugf("'swagger generate server --spec=api-temp.yaml' returned the following output: %s", string(output))

	// TODO: execute 'go get ./...'

	return nil
}
